/**
 * @file
 * Provides mobile toggler for the the Ultimenu main block.
 */

(function ($, Drupal, drupalSettings, _win, _doc, $HTML) {

  'use strict';

  var NAME = 'ultimenu';
  var IS_NAME = 'is-' + NAME;
  var IS_ULTI = 'is-ulti';
  var IS_ROOT_DESKTOP = IS_ULTI + 'desktop';
  var IS_ROOT_MOBILE = IS_ULTI + 'mobile';
  var IS_ROOT_EXPANDED = IS_NAME + '--expanded';
  var IS_ROOT_ACTIVE = IS_NAME + '--active';
  var IS_ROOT_HOVER = IS_NAME + '--hover';
  var IS_ROOT_HIDING = IS_NAME + '--hiding';
  // @todo also replace these for BEM later, not crucial for now.
  var IS_ITEM_EXPANDED = IS_NAME + '-item-expanded';
  var IS_LINK_ACTIVE = IS_NAME + '-active';
  var IS_HAMBURGER_ACTIVE = IS_NAME + '-button-active';
  var IS_AJAX_HIT = 'data-ultiajax-hit';
  var IS_HOVER = false;
  var C_HIDDEN = 'hidden';
  var C_FLYOUT = 'flyout';
  var IS_FLYOUT = 'is-' + C_FLYOUT;
  var IS_FLYOUT_EXPANDED = IS_FLYOUT + '-expanded';
  var C_LINK = NAME + '__link';
  var S_LINK = '.' + C_LINK;
  var S_AJAX_LINK = '.' + NAME + '__ajax';
  var AJAX_TRIGGER = 'data-ultiajax-trigger';
  var C_ULTIMENU_FLYOUT = NAME + '__' + C_FLYOUT;
  var C_OFFCANVAS = IS_NAME + '__canvas-off';
  var C_ONCANVAS = IS_NAME + '__canvas-on';
  var C_BACKDROP = IS_NAME + '__backdrop';
  var C_CARET = NAME + '__caret';
  var S_CARET = '.' + C_CARET;
  var S_BACKDROP = '.' + C_BACKDROP;
  var S_HAMBURGER = '[data-' + NAME + '-button]';
  var S_HAMBURGER_MAIN = '[data-ultimenu-button="#ultimenu-main"]';
  var TIMER_HIDING = void 0;
  var TIMER_WAITING = void 0;
  var ADDCLASS = 'addClass';
  var REMOVECLASS = 'removeClass';
  var E_DBLCLICK = 'dblclick';
  var SETTINGS = drupalSettings.ultimenu || {};
  var WIN_SIZE = {};

  Drupal.ultimenu = {
    $backdrop: null,
    $hamburger: null,
    $offCanvas: null,
    breakpoint: '768px',
    isInvalid: function (e, selector) {
      return !$.is(e.target, selector) || e.type === E_DBLCLICK;
    },

    breakpointChange: function () {
      var me = this;
      var mediaQuery = _win.matchMedia('(max-width: ' + me.breakpoint + ')');

      var removeClasses = function () {
        $.removeClass($HTML, [
          IS_ROOT_ACTIVE,
          IS_ROOT_HOVER,
          IS_ROOT_EXPANDED
        ]);
        me.closeFlyout();
      };

      var toggleClasses = function (e) {
        me.viewport();
        removeClasses();

        if (e.matches) {
          $.addClass($HTML, [IS_ROOT_MOBILE, IS_ROOT_ACTIVE]);
          $.removeClass($HTML, IS_ROOT_DESKTOP);
        }
        else {
          // Do not cache the selector, to avoid incorrect classes.
          // If hamburger is hidden, meaning hover.
          if (me.isHidden($.find(_doc, S_HAMBURGER) || me.$hamburger)) {
            $.addClass($HTML, IS_ROOT_HOVER);
          }
          else {
            $.addClass($HTML, IS_HOVER ? IS_ROOT_HOVER : IS_ROOT_ACTIVE);
          }

          $.addClass($HTML, IS_ROOT_DESKTOP);
          $.removeClass($HTML, IS_ROOT_MOBILE);
        }
      };

      toggleClasses(mediaQuery);
      mediaQuery.addEventListener('change', toggleClasses);
    },

    onClickHamburger: function (e) {
      var me = this;

      e.preventDefault();
      e.stopPropagation();

      var $button = e.target;
      var expanded = $.hasClass($HTML, IS_ROOT_EXPANDED);

      if (me.isInvalid(e, S_HAMBURGER)) {
        return false;
      }

      _win.setTimeout(function () {
        $[expanded ? REMOVECLASS : ADDCLASS]($HTML, IS_ROOT_EXPANDED);
        $[expanded ? REMOVECLASS : ADDCLASS]($button, IS_HAMBURGER_ACTIVE);

        me.closeFlyout();
      }, 30);

      // Cannot use transitionend as can be jumpy affected by child transitions.
      if (expanded) {
        _win.clearTimeout(TIMER_HIDING);
        $.addClass($HTML, IS_ROOT_HIDING);

        TIMER_HIDING = _win.setTimeout(function () {
          $.removeClass($HTML, IS_ROOT_HIDING);
        }, 500);
      }

      // Scroll to top in case the current viewport is far below the fold.
      /*
      if (me.$backdrop && !expanded) {
        _win.scroll({
          top: me.$backdrop.offsetTop,
          behavior: 'smooth'
        });
      }
      */
    },

    viewport: function () {
      var me = this;

      WIN_SIZE = $.windowSize();
      var vh = WIN_SIZE.height / 100;

      $HTML.style.setProperty('--vh', vh + 'px');

      var style = _win.getComputedStyle($HTML);
      me.breakpoint = style.getPropertyValue('--ultimobile');
    },

    offcanvas: function () {
      var me = this;

      // @todo remove BC.
      var header = $.find(_doc, 'is-ultimenu-canvas-off');
      if (header) {
        $.addClass(header, C_OFFCANVAS);
      }

      var siblings = $.findAll(_doc, 'is-ultimenu-canvas-on');
      if (siblings.length) {
        $.addClass(siblings, C_ONCANVAS);
      }

      me.$offCanvas = $.find(_doc, '.' + C_OFFCANVAS);

      if (SETTINGS.canvasOff && SETTINGS.canvasOn) {
        if (!me.$offCanvas) {
          me.$offCanvas = $.find(_doc, SETTINGS.canvasOff);
          $.addClass(me.$offCanvas, C_OFFCANVAS);
        }

        var $onCanvas = $.find(_doc, '.' + C_ONCANVAS);
        if (!$onCanvas) {
          var $onCanvases = $.findAll(_doc, SETTINGS.canvasOn);
          $.addClass($onCanvases, C_ONCANVAS);
        }
      }
    },

    backdrop: function () {
      var me = this;

      // @todo remove BC.
      var backdrop = $.find(_doc, 'is-ultimenu-canvas-backdrop');
      if (backdrop) {
        $.addClass(backdrop, C_BACKDROP);
      }

      me.$backdrop = $.find(_doc, S_BACKDROP);

      if (!me.$backdrop && me.$offCanvas) {
        var $parent = me.$offCanvas.parentNode;
        var el = _doc.createElement('div');
        el.className = C_BACKDROP;
        $parent.insertBefore(el, $parent.firstElementChild || null);

        me.$backdrop = el;
      }

      if (me.$backdrop) {
        $.on(me.$backdrop, 'click.' + NAME, me.onClickBackdrop.bind(me));
      }
    },

    hamburger: function () {
      var me = this;

      if ($.find(_doc, 'body > ' + S_HAMBURGER_MAIN) === null) {
        var hamburger = $.find(_doc, S_HAMBURGER_MAIN);
        if (hamburger) {
          _doc.body.appendChild(hamburger);
        }
      }

      me.$hamburger = $.find(_doc, S_HAMBURGER);

      // Reacts on clicking Ultimenu hamburger button.
      $.on(me.$hamburger, 'click.' + NAME, me.onClickHamburger.bind(me));
    },

    slideToggle: function (el, className) {
      if (el) {
        $[el.clientHeight === 0 ? ADDCLASS : REMOVECLASS](el, className);
      }
    },

    onResize: function () {
      var me = this;

      me.viewport();
    },

    executeAjax: function (el) {
      var $li = $.closest(el, 'li');
      var $ajax = $.find($li, S_AJAX_LINK);

      var cleanUp = function () {
        // Removes attribute to prevent this event from firing again.
        el.removeAttribute(AJAX_TRIGGER);
      };

      // The AJAX link will be gone on successful AJAX request.
      if ($ajax) {
        // Hover event can fire many times, prevents from too many clicks.
        if (!$ajax.hasAttribute(IS_AJAX_HIT)) {
          $ajax.click();

          $ajax.setAttribute(IS_AJAX_HIT, 1);
          $.addClass($ajax, C_HIDDEN);
        }

        // This is the last resort while the user is hovering over menu link.
        // If the AJAX link is still there, an error likely stops it, or
        // the AJAX is taking longer time than 1.5 seconds. In such a case,
        // TIMER_WAITING will re-fire the click event, yet on interval now.
        // At any rate, Drupal.Ajax.ajaxing manages the AJAX requests.
        _win.clearTimeout(TIMER_WAITING);
        TIMER_WAITING = _win.setTimeout(function () {
          $ajax = $.find($li, S_AJAX_LINK);
          if ($ajax) {
            $.removeClass($ajax, C_HIDDEN);
            $ajax.click();
          }
          else {
            cleanUp();
          }

          var onces = $.findAll($li, '[data-once]');
          if (onces.length) {
            $.each(onces, function (elOnce) {
              Drupal.attachBehaviors(elOnce);
            });
          }
        }, 1500);
      }
      else {
        cleanUp();
      }
    },

    triggerAjax: function (e) {
      var me = this;
      var target = e.target;

      e.stopPropagation();

      var link = $.hasClass(target, C_CARET) ?
        $.closest(target, S_LINK) : target;

      if ($.hasAttr(link, AJAX_TRIGGER)) {
        me.executeAjax(link);
      }
    },

    onClickBackdrop: function (e) {
      var me = this;

      e.preventDefault();

      if (me.isInvalid(e, S_BACKDROP)) {
        return false;
      }

      if (me.$hamburger) {
        me.$hamburger.click();
      }
    },

    closeFlyout: function () {
      var actives = $.findAll(_doc, '.' + IS_LINK_ACTIVE);
      var expands = $.findAll(_doc, '.' + IS_ITEM_EXPANDED);
      var flyouts = $.findAll(_doc, '.' + IS_FLYOUT_EXPANDED);

      $.removeClass(actives, IS_LINK_ACTIVE);
      $.removeClass(expands, IS_ITEM_EXPANDED);
      $.removeClass(flyouts, IS_FLYOUT_EXPANDED);
    },

    isHidden: function (el) {
      if (el) {
        if (el.offsetParent === null || el.clientHeight === 0) {
          return true;
        }
        var style = _win.getComputedStyle(el);
        return style.display === 'none' || style.visibility === 'hidden';
      }
      return false;
    },

    onClickCaret: function (e) {
      var me = this;
      var $caret = e.target;

      e.preventDefault();
      e.stopPropagation();

      if (me.isInvalid(e, S_CARET)) {
        return false;
      }

      var $link = $.closest($caret, S_LINK);
      var $li = $.closest($link, 'li');
      var $flyout = $link.nextElementSibling;

      // If hoverable for desktop, one at a time click should hide flyouts.
      // We let regular mobile toggle not affected, to avoid jumping accordion.
      if (me.isHidden(me.$hamburger)) {
        me.closeFlyout();
      }

      // Toggle the current flyout.
      if ($flyout && $.hasClass($flyout, C_ULTIMENU_FLYOUT)) {
        var hidden = me.isHidden($flyout);

        $[hidden ? ADDCLASS : REMOVECLASS]($li, IS_ITEM_EXPANDED);
        $[hidden ? ADDCLASS : REMOVECLASS]($link, IS_LINK_ACTIVE);

        me.slideToggle($flyout, IS_FLYOUT_EXPANDED);
      }
    },

    prepare: function () {
      var me = this;

      IS_HOVER = $.hasClass($HTML, IS_ROOT_HOVER);

      // Checks viewport height based to set var(--vh).
      $.resize(me.onResize.bind(me))();

      // Allows hard-coded CSS classes to not use this.
      me.offcanvas();

      // Prepends our backdrop before the main off-canvas element.
      me.backdrop();

      // Moves the hamburger button to the end of the body.
      me.hamburger();

      // Reacts on resizing, scroll and breakpoint changes.
      me.breakpointChange();
    }

  };

})(dBlazy, Drupal, drupalSettings, this, this.document, this.document.documentElement);
