/**
 * @file
 * Provides mobile toggler for the the Ultimenu main block.
 */

(function ($, Drupal) {

  'use strict';

  var NAME = 'ultimenu';
  var ID_ONCE = NAME;
  var C_MOUNTED = NAME + '--on';
  var S_BASE = '[data-' + NAME + ']';
  var S_ELEMENT = S_BASE + ':not(.' + C_MOUNTED + ')';

  Drupal.ultimenu = Drupal.ultimenu || {};

  /**
   * Ultimenu utility functions for the main menu only.
   *
   * @param {HTMLElement} elm
   *   The ultimenu HTML element.
   */
  function process(elm) {
    var me = Drupal.ultimenu;

    // Applies to other Ultimenus.
    $.on(elm, 'click', '.ultimenu__caret', me.onClickCaret.bind(me));

    $.addClass(elm, C_MOUNTED);
  }

  /**
   * Attaches Ultimenu behavior to HTML element [data-ultimenu].
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.ultimenu = {
    attach: function (context) {

      var me = Drupal.ultimenu;
      var ctx = $.context(context, S_BASE);
      var items = $.findAll(ctx, S_ELEMENT);

      if (items.length) {
        me.prepare();

        $.once(process, ID_ONCE, S_ELEMENT, context);
      }
    },
    detach: function (context, setting, trigger) {
      if (trigger === 'unload') {
        $.once.removeSafely(ID_ONCE, S_ELEMENT, context);
      }
    }
  };

})(dBlazy, Drupal);
